#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "HitLoc.h"
#include "PalmTech.h"
#include "Resource.h"
#include "Misc.h"

#define TEXT_X      15
#define TEXT_Y      35

// ************************************************************* //

Int g_iCriticals=0;
Byte g_pbCriticalList [12];
HitLocation g_hlLocalHitLocation=HIT_NONE;

// ************************************************************* //

void GenerateCriticalList ();
void DrawCriticalList (Boolean bErase);
Int GetSections ();

// ************************************************************* //

Boolean CriticalHitProc(EventPtr event)
{
    Int iHandled = 0;

    switch (event->eType) 
    {
        case frmOpenEvent:
        {
            FormPtr fpForm;

            fpForm = FrmGetActiveForm();
            
            if (g_iCriticals < 0)
            {
                if ((g_hlLocation == HIT_CENTER_CRIT) ||
                    (g_hlLocation == HIT_LEFT_CRIT) ||
                    (g_hlLocation == HIT_RIGHT_CRIT))
                {
                    g_hlLocalHitLocation = (g_hlLocation - HIT_CENTER_CRIT) + HIT_CENTER_TORSO;
                }
                else
                {
                    g_hlLocalHitLocation = g_hlLocation;
                }
                GenerateCriticalList();
            }

            InitPopup (fpForm, IDC_SECTION_TRIGGER, 
                IDC_SECTION_LIST, g_hlLocalHitLocation - HIT_HEAD);
            
            FrmDrawForm(fpForm);
            DrawCriticalList(false);
            iHandled = 1;
            break;
        }
    
        case ctlSelectEvent:  // A control button was pressed and released.
            switch (event->data.ctlEnter.controlID)
            {
                case IDC_BUTTON_ROLL:
                    DrawCriticalList(true);
                    GenerateCriticalList();
                    DrawCriticalList(false);
                    break;
                case IDC_SECTION_TRIGGER:
                    DrawCriticalList(true);
                    g_hlLocalHitLocation = SelectPopup (FrmGetActiveForm(), 
                        IDC_SECTION_TRIGGER, IDC_SECTION_LIST, 
                        g_hlLocalHitLocation - HIT_HEAD) + HIT_HEAD;
                    break;
                case IDC_BUTTON_NEXT:
                    GotoNextForm();
                    break;
                case IDC_BUTTON_PREV:
                    GotoPrevForm();
                    break;
            }
            iHandled = 1;
            break;

        case menuEvent:
            switch (event->data.menu.itemID)
            {
                default:
                    HandleCommonMenu (event->data.menu.itemID);
            }
            iHandled = 1;
            break;
    }

    return iHandled;
}

void GenerateCriticalList ()
{
    DieRoll drRoll;
    Long lRoll;
    Int i;

    drRoll.iNumber = 2;
    drRoll.iDie = 6;
    drRoll.iModifier = 0;
    drRoll.iBest = 0;
    lRoll = RollDie(drRoll);
    if (lRoll < 8)
        g_iCriticals = 0;
    else if (lRoll < 10)
        g_iCriticals = 1;
    else if (lRoll < 12)
        g_iCriticals = 2;
    else 
        g_iCriticals = 3;

    if (g_iCriticals)
    {
        for (i=0; i<12; i++)
            g_pbCriticalList [i] = i;
        QShuffle (g_pbCriticalList, 6 * GetSections(), sizeof(Byte));
    }
}

void DrawCriticalList (Boolean bErase)
{
    Int iCurrentX=TEXT_X, iCurrentY=TEXT_Y, iTemp, i;
    Int iCharHeight;
    Char szText [50];
    Boolean bDrawChart=false;

    if (g_hlLocalHitLocation == HIT_NONE)
        return;
    iCharHeight = FntCharHeight();

 //   DrawHitLocation(bErase, iCurrentX, iCurrentY);
 //   iCurrentY += iCharHeight;
    if (bErase)
    {
        RectangleType rtRect;

        rtRect.topLeft.x = 0;
        rtRect.topLeft.y = iCurrentY;
        rtRect.extent.x = 160;
        rtRect.extent.y = (iCurrentY + (iCharHeight * 6)) + 2;
 
        WinEraseRectangle(&rtRect, 0);

        return;
    }
    switch (g_iCriticals)
    {
        case 0:
            SysCopyStringResource(szText, IDS_NO_CRITICAL);
            break;
        case 3:
            if ((g_hlLocalHitLocation == HIT_HEAD) ||
                (g_hlLocalHitLocation == HIT_LEFT_ARM) || 
                (g_hlLocalHitLocation == HIT_RIGHT_ARM) || 
                (g_hlLocalHitLocation == HIT_LEFT_LEG) || 
                (g_hlLocalHitLocation == HIT_RIGHT_LEG))
            {
                SysCopyStringResource(szText, (Word)((g_hlLocalHitLocation - HIT_HEAD) + IDS_HEAD));
                SysCopyStringResource(&szText[StrLen(szText)], IDS_CRITICAL_OFF);
                break;
            }
        case 2:
        case 1:
            StrIToA(szText, g_iCriticals);
            SysCopyStringResource(&szText[StrLen(szText)], IDS_CRITICAL_HIT);
            if (g_iCriticals != 1)
                StrCat(szText, "s");
            bDrawChart=true;
            break;
    }

    WinDrawChars(szText, StrLen(szText), iCurrentX, iCurrentY);

    if (bDrawChart)
    {
        iCurrentY += iCharHeight+2;
        for (i=0; i<6 * GetSections(); i++)
        {
            iTemp = g_pbCriticalList [i];
            SysCopyStringResource(szText, (iTemp / 6) ? IDS_SECTION2 : IDS_SECTION1);
            StrIToA(&szText[StrLen(szText)], (iTemp % 6) + 1);
            WinDrawChars(szText, StrLen(szText), 
                iCurrentX + (70 * (i / 6)), 
                iCurrentY + (iCharHeight * (i % 6)));
        }
    }
}

Int GetSections ()
{     
    if ((g_hlLocalHitLocation == HIT_HEAD) || 
        (g_hlLocalHitLocation == HIT_LEFT_LEG) || 
        (g_hlLocalHitLocation == HIT_RIGHT_LEG))
    {
        return 1;
    }
 
    return 2;
}
