#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "CritHit.h"
#include "PalmTech.h"
#include "HitLoc.h"
#include "Misc.h"
#include "Resource.h"

#define TEXT_X      10
#define TEXT_Y      50

// ************************************************************* //
HitLocation hlNormalFrontRearLocation [] =
{
    HIT_CENTER_CRIT,
    HIT_RIGHT_ARM,
    HIT_RIGHT_ARM,
    HIT_RIGHT_LEG,
    HIT_RIGHT_TORSO,
    HIT_CENTER_TORSO,
    HIT_LEFT_TORSO,
    HIT_LEFT_LEG,
    HIT_LEFT_ARM,
    HIT_LEFT_ARM,
    HIT_HEAD
};
HitLocation hlNormalLeftLocation [] =
{
    HIT_LEFT_CRIT,
    HIT_LEFT_LEG,
    HIT_LEFT_ARM,
    HIT_LEFT_ARM,
    HIT_LEFT_LEG,
    HIT_LEFT_TORSO,
    HIT_CENTER_TORSO,
    HIT_RIGHT_TORSO,
    HIT_RIGHT_ARM,
    HIT_RIGHT_LEG,
    HIT_HEAD
};
HitLocation hlNormalRightLocation [] =
{
    HIT_RIGHT_CRIT,
    HIT_RIGHT_LEG,
    HIT_RIGHT_ARM,
    HIT_RIGHT_ARM,
    HIT_RIGHT_LEG,
    HIT_RIGHT_TORSO,
    HIT_CENTER_TORSO,
    HIT_LEFT_TORSO,
    HIT_LEFT_ARM,
    HIT_LEFT_LEG,
    HIT_HEAD
};
// ************************************************************* //
HitLocation hlKickFrontRearLocation [] = 
{
    HIT_RIGHT_LEG,
    HIT_LEFT_LEG
};
HitLocation hlKickLeftLocation [] = 
{
    HIT_LEFT_LEG,
    HIT_LEFT_LEG
};
HitLocation hlKickRightLocation [] = 
{
    HIT_RIGHT_LEG,
    HIT_RIGHT_LEG
};
// ************************************************************* //
HitLocation hlPunchFrontRearLocation [] = 
{
    HIT_LEFT_ARM,
    HIT_LEFT_TORSO,
    HIT_CENTER_TORSO,
    HIT_RIGHT_TORSO,
    HIT_RIGHT_ARM,
    HIT_HEAD,
};
HitLocation hlPunchLeftLocation [] = 
{
    HIT_LEFT_TORSO,
    HIT_LEFT_TORSO,
    HIT_CENTER_TORSO,
    HIT_LEFT_ARM,
    HIT_LEFT_ARM,
    HIT_HEAD,
};
HitLocation hlPunchRightLocation [] = 
{
    HIT_RIGHT_TORSO,
    HIT_RIGHT_TORSO,
    HIT_CENTER_TORSO,
    HIT_RIGHT_ARM,
    HIT_RIGHT_ARM,
    HIT_HEAD,
};
// ************************************************************* //
HitLocation hlAboveLocation [] = 
{
    HIT_LEFT_ARM,
    HIT_LEFT_TORSO,
    HIT_CENTER_TORSO,
    HIT_RIGHT_TORSO,
    HIT_RIGHT_ARM,
    HIT_HEAD,
};
HitLocation hlBelowLocation [] = 
{
    HIT_LEFT_LEG,
    HIT_LEFT_LEG,
    HIT_LEFT_TORSO,
    HIT_RIGHT_TORSO,
    HIT_RIGHT_LEG,
    HIT_RIGHT_LEG,
};
// ************************************************************* //

HitChart hcFrontRearChart [] =
{
    2, 6, (HitLocationPtr)hlNormalFrontRearLocation,
    1, 6, (HitLocationPtr)hlPunchFrontRearLocation,
    1, 2, (HitLocationPtr)hlKickFrontRearLocation,
    1, 6, (HitLocationPtr)hlAboveLocation, 
    1, 6, (HitLocationPtr)hlBelowLocation
};
HitChart hcLeftChart [] =
{
    2, 6, (HitLocationPtr)hlNormalLeftLocation,
    1, 6, (HitLocationPtr)hlPunchLeftLocation,
    1, 2, (HitLocationPtr)hlKickLeftLocation,
    1, 6, (HitLocationPtr)hlAboveLocation, 
    1, 6, (HitLocationPtr)hlBelowLocation
};
HitChart hcRightChart [] =
{
    2, 6, (HitLocationPtr)hlNormalRightLocation,
    1, 6, (HitLocationPtr)hlPunchRightLocation,
    1, 2, (HitLocationPtr)hlKickRightLocation,
    1, 6, (HitLocationPtr)hlAboveLocation, 
    1, 6, (HitLocationPtr)hlBelowLocation
};
// ************************************************************* //

Int g_iSide=0, g_iLocation=0;
HitLocation g_hlLocation=HIT_NONE;
char szText [50];

#ifdef _DEBUG
Int iDie, iNumber, iRoll;
#endif

// ************************************************************* //

void GenerateHitLocation ();

// ************************************************************* //

Boolean HitLocationProc(EventPtr event)
{
    Int iHandled = 0;

    switch (event->eType) 
    {
        case frmOpenEvent:
        {
            FormPtr fpForm;

            fpForm = FrmGetActiveForm();

            InitPopup (fpForm, IDC_SIDE_TRIGGER, IDC_SIDE_LIST, g_iSide);
            InitPopup (fpForm, IDC_LOCATION_TRIGGER, IDC_LOCATION_LIST, g_iLocation);

            FrmDrawForm(fpForm);
            DrawHitLocation(false, TEXT_X, TEXT_Y);

            iHandled = 1;
            break;
        }
    
        case ctlSelectEvent:  // A control button was pressed and released.
            switch (event->data.ctlEnter.controlID)
            {
                case IDC_BUTTON_ROLL:
                    DrawHitLocation(true, TEXT_X, TEXT_Y);
                    GenerateHitLocation();
                    DrawHitLocation(false, TEXT_X, TEXT_Y);
                    break;
                case IDC_SIDE_TRIGGER:
                    g_iSide = SelectPopup (FrmGetActiveForm(), 
                        IDC_SIDE_TRIGGER, IDC_SIDE_LIST, g_iSide);
                    break;
                case IDC_LOCATION_TRIGGER:
                    g_iLocation = SelectPopup (FrmGetActiveForm(), 
                        IDC_LOCATION_TRIGGER, IDC_LOCATION_LIST, g_iLocation);
                    break;
                case IDC_BUTTON_NEXT:
                    GotoNextForm();
                    break;
                case IDC_BUTTON_PREV:
                    GotoPrevForm();
                    break;
            }
            iHandled = 1;
            break;

        case menuEvent:
            switch (event->data.menu.itemID)
            {
                default:
                    HandleCommonMenu (event->data.menu.itemID);
            }
            iHandled = 1;
            break;
    }

    return iHandled;
}

void GenerateHitLocation ()
{
    DieRoll drRoll;
    HitChartPtr phcChart;

    switch (g_iSide)
    {
        case SIDE_FRONT:
        case SIDE_REAR:
            phcChart = &hcFrontRearChart [g_iLocation];
            break;
        case SIDE_LEFT:
            phcChart = &hcLeftChart [g_iLocation];
            break;
        case SIDE_RIGHT:
            phcChart = &hcRightChart [g_iLocation];
            break;
    }
    drRoll.iNumber = phcChart->iNumber;
    drRoll.iDie = phcChart->iDie;
    drRoll.iModifier = 0;
    drRoll.iBest = 0;
#ifdef _DEBUG
    iNumber = phcChart->iNumber;
    iDie = phcChart->iDie;
    iRoll = RollDie(drRoll);

    g_hlLocation = phcChart->phlLocations [iRoll-phcChart->iNumber];
#else
    g_hlLocation = phcChart->phlLocations [RollDie(drRoll)-phcChart->iNumber];
#endif
    if ((g_hlLocation == HIT_CENTER_CRIT) ||
        (g_hlLocation == HIT_LEFT_CRIT) ||
        (g_hlLocation == HIT_RIGHT_CRIT))
        g_iCriticals = -1;
}

void DrawHitLocation (Boolean bErase, Int iX, Int iY)
{
    Boolean bCrit = false;
    HitLocation hlTemp;

    if (g_hlLocation == HIT_NONE)
        return;

    hlTemp = g_hlLocation;
    if ((hlTemp == HIT_CENTER_CRIT) ||
        (hlTemp == HIT_LEFT_CRIT) ||
        (hlTemp == HIT_RIGHT_CRIT))
    {
        hlTemp = (hlTemp - HIT_CENTER_CRIT) + HIT_CENTER_TORSO;
        bCrit = true;
    }
    SysCopyStringResource(szText, (Word)((hlTemp - HIT_HEAD) + IDS_HEAD));
    if (g_iSide == SIDE_REAR)
    {
        if ((hlTemp == HIT_CENTER_TORSO) ||
            (hlTemp == HIT_LEFT_TORSO) ||
            (hlTemp == HIT_RIGHT_TORSO))
        {
            StrCat(szText, " (");
            SysCopyStringResource(&szText[StrLen(szText)], IDS_REAR);
            StrCat(szText, ")");
        }
    }
    if (bCrit)
    {
        StrCat(szText, " *");
        SysCopyStringResource(&szText[StrLen(szText)], IDS_CRITICAL);
    }
#ifdef _DEBUG
    StrCat(szText, " - ");
    StrIToA(&szText[StrLen(szText)], iNumber);
    StrCat(szText, "d");
    StrIToA(&szText[StrLen(szText)], iDie);
    StrCat(szText, " - ");
    StrIToA(&szText[StrLen(szText)], iRoll);
#endif

    if (bErase)
    {
        RectangleType rtRect;

        rtRect.topLeft.x = iX;
        rtRect.topLeft.y = iY;
        rtRect.extent.x = 160 - iX;
        rtRect.extent.y = FntCharHeight();
 
        WinEraseRectangle(&rtRect, 0);

        return;
    }
    else
    {
        WinDrawChars(szText, StrLen(szText), iX, iY);
    }
}
